/*

various functions

*/
// includes

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include <util/atomic.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "main.h"
#include <util/delay.h>
#include "globals.h"
#include "uart.h"
#include "functions.h"

int putchar__(char c, FILE *stream) {
	uart_putc(c);
	return 0;
}

uint8_t spi_io(uint8_t data) {
    SPDR = data;
    while(!(SPSR & (1<<SPIF)));
    return SPDR;
}

// atomic access to software timebase

uint32_t get_timebase(void){
    uint32_t time;

    ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
        time = gtimebase;
    }
    return time;
};

void set_TLC_clock(uint8_t prescaler_mask) {

    switch(prescaler_mask) {
        case 0: OCR1A = 0; break;
        case 1: OCR1A = 0; break;
        case 3: OCR1A = 0; break;
        case 7: OCR1A = 1; break;
    }
}

void clear_event(void) {
    uint8_t i;

    for(i=0; i<16; i++) {
        gevent_data[i]=NOP_LED;
    }
}

// map led number from user count 1-16 into internal count 0-15

uint8_t led_map(uint8_t led) {

    if(led<9) {
        return ((led-1)*2);
    } else {
        return ((led-9)*2+1);
    }
}

void cfg_read(void) {
    eeprom_read_block(&gconfig, &gconfig_ee, sizeof(gconfig));
}

void cfg_write(void) {
    eeprom_update_block(&gconfig, &gconfig_ee, sizeof(gconfig));
}
