#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <string.h>
#include <stdlib.h>
#include "main.h"
#include <util/delay.h>
#include "globals.h"
#include "uart.h"

// timer 0 overflow interrupt
// 8kHz PWM timebase

ISR(TIMER0_OVF_vect) {
    static uint8_t prescaler;
    static uint8_t event_old, event_act;
    uint8_t event_new, event_new1;
    static uint16_t event_debounce;

    prescaler--;
    if (!(prescaler & gconfig.pwm_prescaler_mask)) {
        gflag_pwm = 1;

        if (gflag_new_event) {    
            gflag_new_event = 0;
            XLAT_HIGH                                           // store new PWM data
            XLAT_LOW            
        } else {
            _delay_us(0.5);       // compensate run time of else branch
        }

        BLANK_HIGH      // reset TLC5940 PWM logic
        BLANK_LOW

        // debounce event port

        // microdebounce

        event_new = TRIGGER_IN;
        event_new1 = TRIGGER_IN;
        if (event_new1 != event_new) {
            event_new = TRIGGER_IN;
        }

        // makrodebounce

        if (!gconfig.debounce) {
            if (event_new != event_act) {
                event_act = event_new;
                gnext_event = event_new;
                gflag_update = 1;
            }
        } else {
            if ( (event_new == event_old) && (event_new != event_act) ) {
                if (!event_debounce) {
                    event_act = event_new;
                    gnext_event = event_new;
                    gflag_update = 1;
                } else {
                    event_debounce--;
                }
            } else  {
                event_debounce = gconfig.debounce;
            }
        }
        event_old = event_new;
    }

    if (!prescaler) {
        prescaler = 8;
    }

}

// UART RXC interrupt

ISR(USART_RXC_vect) {

    static uint8_t index=0;
    uint8_t tmp;

	tmp=UDR;       						    // get uart data
	if (!gflag_terminal) {
        if (tmp=='\r') {					// CR terminates command
    		gterminal_buffer[index]=0;		// string terminator
    	    index=0;
            gflag_terminal=1;
    	} else {
    		gterminal_buffer[index]=tmp;
    		if (index < (sizeof(gterminal_buffer)-1)) index++;	// prevent overrun
    	}
    }  
}

