/*

Precision Timing LED Module

ATmega8A @16,384 MHz

*/

// includes

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "main.h"
#include <util/delay.h>
#include "globals.h"
#include "uart.h"
#include "functions.h"
#include "cmd_terminal.h"
#include "tlc5940.h"

static FILE mystdout = FDEV_SETUP_STREAM(putchar__, 0, _FDEV_SETUP_WRITE);

// main

int main(void) {

    // variables init

	// redirect stdout for use of PRINT macro
	stdout = &mystdout;

    // IO init
    DDRD  = (1<<PD7) |(1<<PD6) | (1<<PD5) | (1<<PD2) | (1<<PD1);           // TEST, TXD und FLASH_CS
    DDRB  = (1<<PB5) | (1<<PB3) | (1<<PB2) | (1<<PB1) |(1<<PB0);

    FLASH_CS_HIGH
    MUX_STANDALONE
    BLANK_HIGH

    // SPI init

    SPCR = (1<<SPE) | (1<<MSTR);
    SPSR = (1<<SPI2X);

    // UART config

    UCSRB = (1<<RXCIE) | (1<<RXEN) | (1<<TXEN); 
    UBRRH = UBRR_VAL >> 8;
    UBRRL = UBRR_VAL;
    
    // timer 0 init, 8 kHz PWM timebase

    TCCR0 = (1<<CS01);       // XTAL/8
    TIMSK |= (1<<TOIE0); 

    // timer 1 init, PWM clock generator

    TCCR1A = (1<<COM1B0);               // OCRB1 for clock output
    TCCR1B = (1<<WGM12) | (1<<CS10);    // CTC, mode 4, prescaler 1
    OCR1A  =  0;
    OCR1B  =  0;                        // 8,192 MHz PWM clock for TLC5940

    PRINT("\r\nPower Up\r\n");

    // read config

    cfg_read();
    set_TLC_clock(gconfig.pwm_prescaler_mask);

    // init TLC5940

    DC_MODE
    TLC5940_update_DC(gTLC5940_data_DC);
    XLAT_HIGH       // store new data
    XLAT_LOW

    GS_MODE
    TLC5940_update_GS(gTLC5940_data_GS);
    XLAT_HIGH       // store new data
    XLAT_LOW

/*
    // test

    sei();

    int i;

    for (i=0; i<16; i++) {
        TLC5940_set_gs_data(gTLC5940_data_GS, i, 500);
        TLC5940_update_GS(gTLC5940_data_GS);
        gflag_new_event=1;
        _delay_ms(100);

    }
*/
    // configure operation mode

    switch (gconfig.mode) {
        case MODE_EVENT:
            //PRINT("Event mode\r\n");
            MUX_STANDALONE
        break;

        case MODE_BUS:
            //PRINT("Bus Mode\r\n");
            MUX_BUS
        break;

        default:
            //PRINT("Illegal mode defined!\r\nRunning in Config mode\r\n");
            gconfig.mode = MODE_CONFIG;
            MUX_STANDALONE
        break;
    }

    // interrupt enable

    sei();

    // endless main loop

    while (1) {
        if (gflag_terminal) {
            cmd_terminal();
            gflag_terminal=0;
        }
        
        if (gflag_update && (gconfig.mode == MODE_EVENT)) {
            gflag_update=0;
            //TEST_HIGH
            TLC5940_event(gTLC5940_data_GS);
            TLC5940_update_GS(gTLC5940_data_GS);
            gflag_new_event = 1;
            //TEST_LOW
            //PRINT("Event %d\r\n", gnext_event);      // DEBUG
        }   
    }
}
