#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <stdio.h>
#include "main.h"
#include "functions.h"

void flash_write_page(uint8_t *data, uint16_t page, uint16_t count) {
    uint8_t tmp;
    uint32_t adr;
    
    adr = (uint32_t)page << 10;

    FLASH_CS_LOW
    spi_io(0x82);
    spi_io(adr>>16);
    spi_io(adr>>8);
    spi_io(adr>>0);

    count--;
    SPDR = *data++;
    for (; count>0; count--) {
        tmp = *data++;
        while(!(SPSR & (1<<SPIF)));
        SPDR = tmp;
    }
    while(!(SPSR & (1<<SPIF)));
    FLASH_CS_HIGH
}

void flash_read_page(uint8_t *data, uint16_t page, uint16_t count) {
    uint32_t adr;
    uint8_t tmp;
    
    adr = ((uint32_t)page << 10);

    FLASH_CS_LOW
    spi_io(0xD2);
    spi_io(adr>>16);
    spi_io(adr>>8);
    spi_io(adr>>0);
    spi_io(0);
    spi_io(0);
    spi_io(0);
    spi_io(0);

    SPDR = 0;
    for (; count>0; count--) {
        while(!(SPSR & (1<<SPIF)));
        tmp=SPDR;
        SPDR = 0;
        *data++ = tmp;
    }
    while(!(SPSR & (1<<SPIF)));
    FLASH_CS_HIGH
}


uint8_t flash_busy(void) {
    uint8_t status;    

    FLASH_CS_LOW
    spi_io(0xD7);
    status=spi_io(0);
    FLASH_CS_HIGH
    //PRINT("ST:%X ", status);      // DEBUG
    return (~status & 0x80);
}

