#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <stdio.h>
#include "main.h"
#include "globals.h"
#include "functions.h"
#include "serial_flash.h"

void TLC5940_set_dc_data(uint8_t *dc_data, uint8_t channel, uint8_t data) {

    uint8_t i, offset, bits;

    channel = 15-channel;
    bits = channel*6;
    i = bits / 8;
    offset = (bits & 0b111) >>1;
    data  &= 0x3F;

    switch(offset) {
        case 0: dc_data[i] = (dc_data[i] & 0b00000011) | (data <<2); break;
        case 1: dc_data[i] = (dc_data[i] & 0b11111100) | (data >>4);
                dc_data[i+1] = (dc_data[i+1] & 0b00001111) | (data <<4); break;
        case 2: dc_data[i] = (dc_data[i] & 0b11110000) | (data >>2);
                dc_data[i+1] = (dc_data[i+1] & 0b00111111) | (data <<6); break;
        case 3: dc_data[i] = (dc_data[i] & 0b11000000) | data; break;
    }

}

void TLC5940_set_gs_data(uint8_t *gs_data, uint8_t channel, uint16_t data) {

    uint8_t i, offset, bits;

    channel = 15-channel;
    bits = channel*12;
    i = bits / 8;
    offset = bits & 0b111;
    data  &= 0xFFF; 

    if (!offset) {
        gs_data[i] = data >>4;
        gs_data[i+1] = (gs_data[i+1] & 0x0F) | ((data << 4) & 0xF0);
    } else {
        gs_data[i] = (gs_data[i] & 0xF0) | (data >> 8);
        gs_data[i+1] = data & 0xFF;
    }
}

// update TLC5940 DC data

void TLC5940_update_DC(uint8_t *dc_data) {
    uint8_t i, tmp; 

    TLC5940_CS_LOW
    SPDR = dc_data[0];
    for (i=1; i<12; i++) {
        tmp = dc_data[i];
        while(!(SPSR & (1<<SPIF)));
        SPDR = tmp;
    }
    while(!(SPSR & (1<<SPIF)));
    TLC5940_CS_HIGH
}

// update TLC5940 GS data

void TLC5940_update_GS(uint8_t *gs_data) {
    uint8_t i, tmp; 

    TLC5940_CS_LOW
    spi_io(gconfig.imax);           // software Imax

    SPDR = gs_data[0];
    for (i=1; i<24; i++) {
        tmp = gs_data[i];
        while(!(SPSR & (1<<SPIF)));
        SPDR = tmp;
    }
    while(!(SPSR & (1<<SPIF)));
    TLC5940_CS_HIGH
}


// load data from flash and update TLC5940 data

void TLC5940_event(uint8_t* data_out) {
    uint8_t i;
    uint16_t data_in[16];

    flash_read_page((uint8_t*)data_in, gnext_event, 32);
    for (i=0; i<16; i++) {
        if (data_in[i] != NOP_LED) {
            TLC5940_set_gs_data(data_out, i, data_in[i]);
        }
    }
}


